package org.cyrano.editor.tool;

import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;

import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.editor.parts.ViewPart;

public class RectangleMouseHandler implements EditPolicyMouseHandler {

  private EditPolicyFeedback editPolicyFeedback;
  private EditPolicyExecutor editPolicyExecutor;

  private ViewPart draggedFg;

  private Point clickDwPt;
  private Point draggedPt;
  private Point clickUpPt;

  private int draggedDx;
  private int draggedDy;

  // --------------------------------------------------------------------------------

  public RectangleMouseHandler(ViewPart draggedFg) {
    this.draggedFg = draggedFg;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void init(//
      EditPolicyFeedback editPolicyFeedback, //
      EditPolicyExecutor editPolicyExecutor) {
    this.editPolicyFeedback = editPolicyFeedback;
    this.editPolicyExecutor = editPolicyExecutor;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void paint(Graphics2D g2d) {
    editPolicyFeedback.paint(g2d, this);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void mouseClickDw(MouseEvent evt) {
    clickDwPt = evt.getPoint();

    draggedDx = clickDwPt.x - draggedFg.getRect().x;
    draggedDy = clickDwPt.y - draggedFg.getRect().y;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void mouseDragged(MouseEvent evt) {
    draggedPt = evt.getPoint();
  }

  // --------------------------------------------------------------------------------

  @Override
  public void mouseClickUp(MouseEvent evt) {
    clickUpPt = evt.getPoint();

    UndoRedoManager.getInstance().execute( //
        editPolicyExecutor.execute(this));
  }

  // --------------------------------------------------------------------------------

  public ViewPart getDraggedFg() {
    return draggedFg;
  }

  // --------------------------------------------------------------------------------

  public Point getClickDwPt() {
    return clickDwPt;
  }

  public Point getDraggedPt() {
    return draggedPt;
  }

  public Point getClickUpPt() {
    return clickUpPt;
  }

  // --------------------------------------------------------------------------------

  public int getDraggedDx() {
    return draggedDx;
  }

  public int getDraggedDy() {
    return draggedDy;
  }
}
