package org.cyrano.timetable.ui.algorithm;

import java.util.Iterator;
import java.util.List;

import org.cyrano.timetable.ui.model.hour.HourBlock;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.utils.Rect;

public class LayoutAlgorithm {

  private final Root timeTable;

  // --------------------------------------------------------------------------------

  public LayoutAlgorithm(Root timeTable) {
    this.timeTable = timeTable;
  }

  // --------------------------------------------------------------------------------

  public void layout(int xoffset, int yoffset, int w, int h) {
    List<Slot> slotList = timeTable.getAllChildrenSlots();

    Iterator<Slot> itt = slotList.iterator();

    while (itt.hasNext()) {
      Slot slot = (Slot) itt.next();

      slot.getRect().set(0, 0, 0, 0);

      if (!slot.getVisible()) {
        itt.remove();
      }
    }

    timeTable.getHourModel().prepare(xoffset, yoffset, w, h, slotList);

    while (!slotList.isEmpty()) {
      Slot next;

      while ((next = nextSlot(slotList)) != null) {
        slotList.remove(next);

        doLayoutSlot(next, true);
      }
    }
  }

  // --------------------------------------------------------------------------------

  private void doLayoutSlot(Slot slot, boolean update) {
    HourBlock hourBlock = timeTable.getHourModel().getHourBlock(slot);

    Rect beg = hourBlock.getBegHour().getRect();
    Rect end = hourBlock.getEndHour().getRect();

    Rect rect = slot.getRect();

    rect.y = beg.y;
    rect.h = end.y - beg.y + end.h;

    rect.x = /*                                 */hourBlock.max(HourBlock.CURRX);
    rect.w = timeTable.getHourModel().getColW() / hourBlock.max(HourBlock.COUNT);

    if (update) {
      hourBlock.set(HourBlock.CURRX, rect.x + rect.w);
    }
  }

  // --------------------------------------------------------------------------------

  private Slot nextSlot(List<Slot> slotList) {
    Slot min = null;

    for (Slot cur : slotList) {
      doLayoutSlot(cur, false);

      if (min == null) {
        min = cur;
        continue;
      }

      if (cur.getRect().x < min.getRect().x) {
        min = cur;
        continue;
      }

      if (cur.getRect().x == min.getRect().x) {
        if (cur.getRect().y < min.getRect().y) {
          min = cur;
        }
      }
    }

    return min;
  }
}
