package org.cyrano.timetable.ui.edit.sect;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.commons.lang.StringUtils;
import org.cyrano.editor.listeners.Command;
import org.cyrano.editor.listeners.UndoRedoManager;
import org.cyrano.timetable.ui.command.CreateChildCommand;
import org.cyrano.timetable.ui.command.UpdateChildCommand;
import org.cyrano.timetable.ui.edit.DlgEditBase;
import org.cyrano.timetable.ui.edit.LabelGetter;
import org.cyrano.timetable.ui.edit.LabelValueBeanComboBoxModel;
import org.cyrano.timetable.ui.main.MainFrame;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;

import com.minotauro.cleda.util.LabelValueBean;

public class DlgEditSect extends DlgEditBase<Sect> {

  private Cour cour;

  private JTextField txtCour;

  private JTextField txtNumb;

  private JComboBox<LabelValueBean<Prof>> cboProf;

  // --------------------------------------------------------------------------------

  public DlgEditSect(JFrame parent, Sect data, Cour cour) {
    super(parent, data, //
        _I18NDlgEditSect.createTitle(), //
        _I18NDlgEditSect.updateTitle());

    this.cour = cour;

    guiFromData(data);
    initCboProf();
  }

  // --------------------------------------------------------------------------------

  private void dataFromGui(Sect sect) {
    sect.setNumb(txtNumb.getText());

    LabelValueBeanComboBoxModel<Prof> dcbm = //
    (LabelValueBeanComboBoxModel<Prof>) cboProf.getModel();

    sect.setProf(dcbm.getSelectedData());
  }

  // --------------------------------------------------------------------------------

  private void guiFromData(Sect sect) {
    String courName = cour != null //
        ? cour.getName()
        : data.getCourRef().getParentRef().getName();

    txtCour.setText(courName);

    if (sect == null) {
      return;
    }

    txtNumb.setText(sect.getNumb());

    LabelValueBeanComboBoxModel<Prof> dcbm = //
    (LabelValueBeanComboBoxModel<Prof>) cboProf.getModel();

    dcbm.setSelectedData(sect.getProf());
  }

  // --------------------------------------------------------------------------------

  @Override
  protected JPanel initUpdatePanel() {
    JPanel ret = new JPanel(new GridBagLayout());

    ret.setBorder(BorderFactory.createTitledBorder(//
        _I18NDlgEditSect.titledBorder()));

    int row = 0;

    GridBagConstraints gbc;

    JLabel lblCour = new JLabel(_I18NDlgEditSect.cour());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(5, 5, 0, 0);
    ret.add(lblCour, gbc);

    txtCour = new JTextField();
    txtCour.setPreferredSize(new Dimension(200, 25));
    txtCour.setEditable(false);
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill = GridBagConstraints.BOTH;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.insets = new Insets(5, 5, 0, 5);
    ret.add(txtCour, gbc);

    // ----------------------------------------

    row++;

    JLabel lblNumb = new JLabel(_I18NDlgEditSect.numb());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 0);
    ret.add(lblNumb, gbc);

    txtNumb = new JTextField();
    txtNumb.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 0, 5);
    ret.add(txtNumb, gbc);

    // ----------------------------------------

    row++;

    JLabel lblProf = new JLabel(_I18NDlgEditSect.prof());
    gbc = initGBC(0, row, 0, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 0);
    ret.add(lblProf, gbc);

    cboProf = new JComboBox<LabelValueBean<Prof>>();
    cboProf.setModel(new LabelValueBeanComboBoxModel<Prof>());
    cboProf.setPreferredSize(new Dimension(200, 25));
    gbc = initGBC(1, row, 1, 0, 1, 1);
    gbc.fill/*  */= GridBagConstraints.BOTH;
    gbc.anchor/**/= GridBagConstraints.WEST;
    gbc.insets/**/= new Insets(5, 5, 5, 5);
    ret.add(cboProf, gbc);

    return ret;
  }

  // --------------------------------------------------------------------------------

  private void initCboProf() {
    Root root = data != null ? data.getRoot() : cour.getRoot();

    LabelValueBeanComboBoxModel<Prof> dcbm = //
    (LabelValueBeanComboBoxModel<Prof>) cboProf.getModel();
    dcbm.removeAllElements();

    dcbm.add(_I18NDlgEditSect.select(), null);

    dcbm.addAll(root.getProfList().getChildrenList(), //
        new LabelGetter<Prof>() {
          @Override
          public String getLabel(Prof data) {
            return data.getInit();
          }
        });

    if (data != null && data.getProf() != null) {
      dcbm.setSelectedData(data.getProf());
    }
  }

  // --------------------------------------------------------------------------------

  @Override
  protected boolean validateData() {
    if (StringUtils.isBlank(txtNumb.getText())) {
      JOptionPane.showMessageDialog(MainFrame.workbenchRef, //
          _I18NDlgEditSect.errorEmptyNumb(), //
          _I18NDlgEditSect.errorTitle(), //
          JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void createBtnAcceptClicked() {
    data = new Sect();
    dataFromGui(data);

    Command command = //
    new CreateChildCommand<Cour, Sect>( //
        cour.getSectList(), data);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }

  // --------------------------------------------------------------------------------

  @Override
  protected void updateBtnAcceptClicked() {
    Sect prevSect = new Sect();
    prevSect.shallowDataCopyFrom(data);

    Sect nextSection = new Sect();
    dataFromGui(nextSection);

    Command command = new UpdateChildCommand( //
        prevSect, data, nextSection);

    UndoRedoManager.getInstance().execute(command);

    dispose();
  }
}
