package org.cyrano.timetable.ui.figures;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;

import org.cyrano.editor.parts.BaseViewPart;
import org.cyrano.editor.parts.PartsRegister;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.utils.ImageCache;
import org.cyrano.utils.Rect;

public class SlotViewPart extends BaseViewPart {

  private static final Font RESTRICTION_VIOLATION_FONT = //
  new Font("Arial", Font.BOLD, 12);

  // --------------------------------------------------------------------------------

  public SlotViewPart(PartsRegister partsRegister) {
    super(partsRegister);
  }

  // --------------------------------------------------------------------------------

  private Slot getSlot() {
    return (Slot) getPartsRegister().getModelByViewPart(this);
  }

  // --------------------------------------------------------------------------------

  @Override
  public void paint(Graphics2D g2d) {
    Slot slot = getSlot();

    if (!slot.getVisible()) {
      return;
    }

    Color back = slot.getBackground();
    Color fore = Color.BLACK; // TODO: Magic!

    Rect rect = slot.getRect();

    if (slot.getSectRef().getParentRef().getRoomViolation() || //
        slot.getSectRef().getParentRef().getProfViolation() || //
        slot.getSectRef().getParentRef().getSectViolation()) {
      back = Color.RED; //   TODO: Magic!
      fore = Color.WHITE; // TODO: Magic!
    }

    g2d.setColor(back);
    g2d.fillRect(rect.x, rect.y, rect.w, rect.h);
    g2d.setColor(fore);
    g2d.drawRect(rect.x, rect.y, rect.w, rect.h);

    Rect shRect = getHandleRect();

    g2d.setColor(fore);
    g2d.drawLine( //
        shRect.x, //
        shRect.y, //
        shRect.x + shRect.w, //
        shRect.y);

    g2d.setClip(rect.x, rect.y, rect.w, rect.h);

    Cour cour = slot. //
        getSectRef().getParentRef(). //
        getCourRef().getParentRef();

    Sect sect = slot. //
        getSectRef().getParentRef();

    String profInit = sect.getProf() != null //
        ? sect.getProf().getInit()
        : "----";
    String roomName = slot.getRoom() != null //
        ? slot.getRoom().getName()
        : "----";

    paintSlotString(g2d, rect.x + 5, rect.y + 15, //
        cour.getName(), false);

    paintSlotString(g2d, rect.x + 5, rect.y + 35, //
        sect.getNumb(), /*      */slot.getSectViolation());

    paintSlotString(g2d, rect.x + 5, rect.y + 55, //
        profInit, slot.getProfViolation());
    paintSlotString(g2d, rect.x + 5, rect.y + 75, //
        roomName, slot.getRoomViolation());

    if (sect.getProf() == null || //
        slot.getRoom() == null) {
      g2d.drawImage(ImageCache.getInstance() //
          .getImage("images/16x16_slot_missing.png"), //
          rect.x + 5, rect.y + 85, null);
    } else if (slot.getRoomViolation() || //
        /*   */slot.getProfViolation() || //
        /*   */slot.getSectViolation()) {
      g2d.drawImage(ImageCache.getInstance() //
          .getImage("images/16x16_slot_problem.png"), //
          rect.x + 5, rect.y + 85, null);
    } else if (slot.getSemsViolation()) {
      g2d.drawImage(ImageCache.getInstance() //
          .getImage("images/16x16_slot_warning.png"), //
          rect.x + 5, rect.y + 85, null);
    }

    g2d.setClip(null);
  }

  // --------------------------------------------------------------------------------

  private void paintSlotString( //
      Graphics2D g2d, int x, int y, String str, boolean violation) {

    Font prevFont = g2d.getFont();

    g2d.setColor(Color.BLACK);

    if (violation) {
      g2d.setColor(Color.WHITE);
      g2d.setFont(RESTRICTION_VIOLATION_FONT);
    }

    g2d.drawString(str, x, y);

    g2d.setFont(prevFont);
  }

  // --------------------------------------------------------------------------------

  public Rect getHandleRect() {
    Rect slotRect = getSlot().getRect();

    int xgap = (int) (slotRect.w * 0.2);
    int ygap = 10;

    xgap = xgap < 5 ? xgap : 5;

    Rect ret = new Rect();

    ret.x = slotRect.minX() + xgap;
    ret.y = slotRect.maxY() - ygap;
    ret.w = slotRect.w - 2 * xgap;
    ret.h = ygap;

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public Rect getRect() {
    return new Rect(getSlot().getRect());
  }
}
