package org.cyrano.timetable.ui.model.timetable;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import org.cyrano.timetable.ui.model.base.BaseModel;
import org.cyrano.timetable.ui.model.base.Children;
import org.cyrano.timetable.ui.model.base.ColorProvider;
import org.cyrano.timetable.ui.model.base.ModelChangeEvent;
import org.cyrano.timetable.ui.model.base.ObjectMatcher;
import org.cyrano.timetable.ui.model.base.VisibilityChangeEvent;

public class Room implements BaseModel, ColorProvider {

  public static final ObjectMatcher<Room, String> ROOM_BY_NAME_MATCHER = //
  new ObjectMatcher<Room, String>() {
    @Override
    public boolean matches(Room object, String target) {
      return target.equals(object.getName());
    }
  };

  // --------------------------------------------------------------------------------

  private Children<Root> rootRef;

  private String name;

  private Color background;

  private boolean visible = true;

  // --------------------------------------------------------------------------------

  public Room() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  public Children<Root> getRootRef() {
    return rootRef;
  }

  public void setRootRef(Children<Root> rootRef) {
    this.rootRef = rootRef;
  }

  // --------------------------------------------------------------------------------

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  // --------------------------------------------------------------------------------
  // ColorProvider
  // --------------------------------------------------------------------------------

  @Override
  public Color getBackground() {
    return background;
  }

  public void setBackground(Color background) {
    this.background = background;
  }

  // --------------------------------------------------------------------------------
  // BaseModel
  // --------------------------------------------------------------------------------

  @Override
  public Root getRoot() {
    return rootRef != null ? rootRef.getParentRef() : null;
  };

  // --------------------------------------------------------------------------------

  @Override
  public String getDisplayName() {
    return name;
  }

  // --------------------------------------------------------------------------------

  public boolean getVisible() {
    return visible;
  }

  @Override
  public void setVisible(boolean visible) {
    this.visible = visible;
    getRoot().getVisibilityChangeSupport(). //
        fireVisibilityChange(new VisibilityChangeEvent(this));
  }

  // --------------------------------------------------------------------------------

  @Override
  public List<Slot> getAllChildrenSlots() {
    List<Slot> ret = new ArrayList<Slot>();

    List<Slot> allChildrenSlots = //
    rootRef.getParentRef().getAllChildrenSlots();

    for (Slot slot : allChildrenSlots) {
      if (slot.getRoom() == this) {
        ret.add(slot);
      }
    }

    return ret;
  }

  // --------------------------------------------------------------------------------

  @Override
  public void shallowDataCopyFrom(BaseModel baseModel) {
    if (!(baseModel instanceof Room)) {
      throw new IllegalArgumentException();
    }

    Room from = (Room) baseModel;

    this.name = from.name;
    this.background = from.background;

    if (getRoot() != null) {
      getRoot().getModelChangeSupport().fireModelChange(new ModelChangeEvent(this, this));
    }
  }
}
