package org.cyrano.timetable.ui.model.visitor;

import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;

public class ModelProcessor {

  private ModelVisitor modelVisitor;

  // --------------------------------------------------------------------------------

  public ModelProcessor( //
      ModelVisitor modelVisitor) {

    this.modelVisitor = modelVisitor;
  }

  // --------------------------------------------------------------------------------

  public void visit(Object object) {
    /*   */if (object instanceof Root) {
      internalVisitRoot((Root) object);
    } else if (object instanceof Sems) {
      internalVisitSems((Sems) object);
    } else if (object instanceof Prof) {
      internalVisitProf((Prof) object);
    } else if (object instanceof Room) {
      internalVisitRoom((Room) object);
    } else if (object instanceof Cour) {
      internalVisitCour((Cour) object);
    } else if (object instanceof Sect) {
      internalVisitSect((Sect) object);
    }
  }

  // --------------------------------------------------------------------------------

  private void internalVisitRoot(Root root) {
    modelVisitor.visitRoot(root);

    for (Sems sems : root.getSemsList().getChildrenList()) {
      internalVisitSems(sems);
    }

    for (Prof prof : root.getProfList().getChildrenList()) {
      internalVisitProf(prof);
    }

    for (Room room : root.getRoomList().getChildrenList()) {
      internalVisitRoom(room);
    }
  }

  // --------------------------------------------------------------------------------

  private void internalVisitSems(Sems sems) {
    modelVisitor.visitSems(sems);

    for (Cour cour : sems.getCourList().getChildrenList()) {
      internalVisitCour(cour);
    }
  }

  // --------------------------------------------------------------------------------

  private void internalVisitProf(Prof prof) {
    modelVisitor.visitProf(prof);
  }

  // --------------------------------------------------------------------------------

  private void internalVisitRoom(Room room) {
    modelVisitor.visitRoom(room);
  }

  // --------------------------------------------------------------------------------

  private void internalVisitCour(Cour cour) {
    modelVisitor.visitCour(cour);

    for (Sect sect : cour.getSectList().getChildrenList()) {
      internalVisitSect(sect);
    }
  }

  // --------------------------------------------------------------------------------

  private void internalVisitSect(Sect sect) {
    modelVisitor.visitSect(sect);

    for (Slot slot : sect.getSlotList().getChildrenList()) {
      modelVisitor.visitSlot(slot);
    }
  }
}
