package org.cyrano.timetable.ui.view;

import java.io.OutputStream;
import java.text.NumberFormat;

import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.timetable.xjc.ObjectFactory;
import org.cyrano.timetable.xjc.TimeTableXmlLoader;
import org.cyrano.timetable.xjc.generated.SlotType;
import org.cyrano.utils.ColorParser;

public class TimeTable2XML {

  private ObjectFactory objectFactory = new ObjectFactory();

  // --------------------------------------------------------------------------------

  public void save(OutputStream os, Root root) throws Exception {
    org.cyrano.timetable.xjc.Root xmlRoot = //
    objectFactory.createRoot();

    saveProfList(xmlRoot, root);
    saveRoomList(xmlRoot, root);

    saveSemsList(xmlRoot, root);

    TimeTableXmlLoader timeTableXmlLoader = new TimeTableXmlLoader();
    timeTableXmlLoader.save(os, xmlRoot);
  }

  // --------------------------------------------------------------------------------

  private void saveProfList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root root) {

    for (Prof prof : root.getProfList().getChildrenList()) {
      org.cyrano.timetable.xjc.UnprefixedProf xmlProf = //
      objectFactory.createProf();

      xmlProf.setUnprefixedInit(prof.getInit());
      xmlProf.setBack(ColorParser.toString(prof.getBackground()));

      xmlRoot.getProf().add(xmlProf);
    }
  }

  // --------------------------------------------------------------------------------

  private void saveRoomList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root root) {

    for (Room room : root.getRoomList().getChildrenList()) {
      org.cyrano.timetable.xjc.UnprefixedRoom xmlRoom = //
      objectFactory.createRoom();

      xmlRoom.setUnprefixedName(room.getName());
      xmlRoom.setBack(ColorParser.toString(room.getBackground()));

      xmlRoot.getRoom().add(xmlRoom);
    }
  }

  // --------------------------------------------------------------------------------

  private void saveSemsList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root root) {

    for (Sems sems : root.getSemsList().getChildrenList()) {
      org.cyrano.timetable.xjc.generated.Sems xmlSems = //
      objectFactory.createSems();

      xmlSems.setName(sems.getName());
      xmlSems.setBack(ColorParser.toString(sems.getBackground()));

      xmlRoot.getSems().add(xmlSems);
      saveCourList(xmlSems, sems);
    }
  }

  // --------------------------------------------------------------------------------

  private void saveCourList( //
      org.cyrano.timetable.xjc.generated.Sems xmlSems, //
      Sems sems) {

    for (Cour cour : sems.getCourList().getChildrenList()) {
      org.cyrano.timetable.xjc.generated.Cour xmlCour = //
      objectFactory.createCour();

      xmlCour.setName(cour.getName());

      xmlSems.getCour().add(xmlCour);
      saveSectList(xmlCour, cour);
    }
  }

  // --------------------------------------------------------------------------------

  private void saveSectList( //
      org.cyrano.timetable.xjc.generated.Cour xmlCour, //
      Cour cour) {

    for (Sect sect : cour.getSectList().getChildrenList()) {
      org.cyrano.timetable.xjc.generated.Sect xmlSect = //
      objectFactory.createSect();

      xmlSect.setNumb(sect.getNumb());

      org.cyrano.timetable.xjc.UnprefixedProf xmlProf = //
      objectFactory.createProf();

      xmlProf.setUnprefixedInit(sect.getProf().getInit());

      xmlSect.setProf(xmlProf);

      xmlCour.getSect().add(xmlSect);
      saveSlotList(xmlSect, sect);
    }
  }

  // --------------------------------------------------------------------------------

  private void saveSlotList( //
      org.cyrano.timetable.xjc.generated.Sect xmlSect, //
      Sect sect) {

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumFractionDigits(0);
    nf.setMaximumFractionDigits(0);
    nf.setMinimumIntegerDigits(2);
    nf.setMaximumIntegerDigits(2);

    for (Slot slot : sect.getSlotList().getChildrenList()) {
      org.cyrano.timetable.xjc.generated.Slot xmlSlot = //
      objectFactory.createSlot();

      xmlSlot.setDay(slot.getDay());
      xmlSlot.setBeg(Integer.parseInt(slot.getBeg()));
      xmlSlot.setEnd(Integer.parseInt(slot.getEnd()));

      org.cyrano.timetable.xjc.UnprefixedRoom xmlRoom = //
      objectFactory.createRoom();

      xmlRoom.setUnprefixedName(slot.getRoom().getName());

      xmlSlot.setRoom(xmlRoom);

      xmlSlot.setType(SlotType.fromValue(slot.getType()));

      xmlSect.getSlot().add(xmlSlot);
    }
  }
}
