package org.cyrano.timetable.ui.view;

import java.io.InputStream;
import java.text.NumberFormat;

import org.cyrano.gimp.palette.SequentialColorMap;
import org.cyrano.timetable.ui.model.hour.HourModel;
import org.cyrano.timetable.ui.model.timetable.Cour;
import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sect;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.timetable.xjc.TimeTableXmlLoader;
import org.cyrano.timetable.xjc.UnprefixedProf;
import org.cyrano.timetable.xjc.UnprefixedRoom;
import org.cyrano.utils.ColorParser;

public class XML2TimeTable {

  public Root load(InputStream is, HourModel hourModel, //
      SequentialColorMap semsCmap, //
      SequentialColorMap profCmap, //
      SequentialColorMap roomCmap) //
      throws Exception {

    TimeTableXmlLoader timeTableXmlLoader = new TimeTableXmlLoader();

    org.cyrano.timetable.xjc.Root xmlRoot = //
    timeTableXmlLoader.load(is);

    Root root = new Root(hourModel);

    loadProfList(xmlRoot, root, profCmap);
    loadRoomList(xmlRoot, root, roomCmap);
    loadSemsList(xmlRoot, root, semsCmap);

    return root;
  }

  // --------------------------------------------------------------------------------

  private void loadProfList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root timeTable, SequentialColorMap profCmap) {

    for (org.cyrano.timetable.xjc.UnprefixedProf xmlProf : //
    /* */xmlRoot.getUnprefixedProf()) {

      Prof prof = new Prof();
      prof.setInit(xmlProf.getUnprefixedInit());

      if (xmlProf.getBack() != null) {
        prof.setBackground(ColorParser.parse(xmlProf.getBack()));
      } else {
        prof.setBackground(profCmap.getColorFor(prof));
      }

      timeTable.getProfList().add(prof);
    }
  }

  // --------------------------------------------------------------------------------

  private void loadRoomList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root timeTable, SequentialColorMap roomCmap) {

    for (org.cyrano.timetable.xjc.UnprefixedRoom xmlRoom : //
    /* */xmlRoot.getUnprefixedRoom()) {

      Room room = new Room();
      room.setName(xmlRoom.getUnprefixedName());

      if (xmlRoom.getBack() != null) {
        room.setBackground(ColorParser.parse(xmlRoom.getBack()));
      } else {
        room.setBackground(roomCmap.getColorFor(room));
      }

      timeTable.getRoomList().add(room);
    }
  }

  // --------------------------------------------------------------------------------

  private void loadSemsList( //
      org.cyrano.timetable.xjc.Root xmlRoot, //
      Root root, SequentialColorMap semsCmap) {

    if (xmlRoot.getSems() == null) {
      return;
    }

    for (org.cyrano.timetable.xjc.generated.Sems xmlSems : //
    /* */xmlRoot.getSems()) {

      Sems sems = root.findSemsByName(xmlSems.getName());

      if (sems == null) {
        sems = new Sems();
        sems.setName(xmlSems.getName());

        if (xmlSems.getBack() != null) {
          sems.setBackground(ColorParser.parse(xmlSems.getBack()));
        } else {
          sems.setBackground(semsCmap.getColorFor(sems));
        }

        root.getSemsList().add(sems);
      }

      loadCourList(xmlSems, sems);
    }
  }

  // --------------------------------------------------------------------------------

  private void loadCourList( //
      org.cyrano.timetable.xjc.generated.Sems xmlSems, //
      Sems sems) {

    if (xmlSems.getCour() == null) {
      return;
    }

    for (org.cyrano.timetable.xjc.generated.Cour xmlCour : //
    /* */xmlSems.getCour()) {

      Cour cour = sems.findCourByName(xmlCour.getName());

      if (cour != null) {
        throw new IllegalStateException("cour != null");
      }

      cour = new Cour();
      cour.setName(xmlCour.getName());
      sems.getCourList().add(cour);

      loadSectList(xmlCour, cour);
    }
  }

  // --------------------------------------------------------------------------------

  private void loadSectList( //
      org.cyrano.timetable.xjc.generated.Cour xmlCour, //
      Cour cour) {

    if (xmlCour.getSect() == null) {
      return;
    }

    for (org.cyrano.timetable.xjc.generated.Sect xmlSect : //
    /* */xmlCour.getSect()) {

      Sect sect = cour.findSectByNumb(xmlSect.getNumb());

      if (sect != null) {
        throw new IllegalStateException("sect != null");
      }

      Root timeTable = cour.getRoot();

      sect = new Sect();
      sect.setNumb(xmlSect.getNumb());

      Prof prof = timeTable.findProfByInit( //
          ((UnprefixedProf) xmlSect.getProf()).getUnprefixedInit());
      sect.setProf(prof);

      cour.getSectList().add(sect);

      loadSlotList(xmlSect, sect);
    }
  }

  // --------------------------------------------------------------------------------

  private void loadSlotList( //
      org.cyrano.timetable.xjc.generated.Sect xmlSect, //
      Sect sect) {

    if (xmlSect.getSlot() == null) {
      return;
    }

    NumberFormat nf = NumberFormat.getInstance();
    nf.setMinimumFractionDigits(0);
    nf.setMaximumFractionDigits(0);
    nf.setMinimumIntegerDigits(2);
    nf.setMaximumIntegerDigits(2);

    for (org.cyrano.timetable.xjc.generated.Slot xmlSlot : //
    /* */xmlSect.getSlot()) {

      String beg = nf.format(xmlSlot.getBeg());
      String end = nf.format(xmlSlot.getEnd());
      String day = xmlSlot.getDay();

      Root timeTable = sect.getRoot();

      Room room = timeTable.findRoomByName( //
          ((UnprefixedRoom) xmlSlot.getRoom()).getUnprefixedName());

      String type = xmlSlot.getType().value();

      sect.getSlotList().add(new Slot(day, beg, end, room, type));
    }
  }
}
