package org.cyrano.timetable.validator.java;

import java.util.List;

import org.cyrano.timetable.ui.model.timetable.Prof;
import org.cyrano.timetable.ui.model.timetable.Room;
import org.cyrano.timetable.ui.model.timetable.Root;
import org.cyrano.timetable.ui.model.timetable.Sems;
import org.cyrano.timetable.ui.model.timetable.Slot;
import org.cyrano.timetable.validator.RestrictionValidator;

public class JavaRestrictionValidator implements RestrictionValidator {

  public JavaRestrictionValidator() {
    // Empty
  }

  // --------------------------------------------------------------------------------

  @Override
  public void validate(Root timeTable) throws Exception {
    List<Slot> slotList = timeTable.getAllChildrenSlots();

    timeTable.clearValidations();

    for (int i = 0; i < slotList.size(); i++) {
      Slot src = slotList.get(i);

      for (int j = i + 1; j < slotList.size(); j++) {
        Slot tgt = slotList.get(j);

        if (src.overlap(tgt)) {
          validateRoom(src, tgt);
          validateProf(src, tgt);
          validateSect(src, tgt);
          validateSems(src, tgt);
        }
      }
    }
  }

  // --------------------------------------------------------------------------------

  private void validateRoom(Slot src, Slot tgt) {
    Room srcRoom = src.getRoom();
    Room tgtRoom = tgt.getRoom();

    if (srcRoom == null || tgtRoom == null) {
      return;
    }

    if (srcRoom.getName().equals(tgtRoom.getName())) {
      src.setRoomViolation(true);
      tgt.setRoomViolation(true);
    }
  }

  // --------------------------------------------------------------------------------

  private void validateProf(Slot src, Slot tgt) {
    Prof srcProf = src.getSectRef().getParentRef().getProf();
    Prof tgtProf = tgt.getSectRef().getParentRef().getProf();

    if (srcProf == null || tgtProf == null) {
      return;
    }

    if (srcProf.getInit().equals(tgtProf.getInit())) {
      src.setProfViolation(true);
      tgt.setProfViolation(true);
    }
  }

  // --------------------------------------------------------------------------------

  private void validateSect(Slot src, Slot tgt) {
    if (src.getSectRef().getParentRef() == //
    /**/tgt.getSectRef().getParentRef()) {
      src.setSectViolation(true);
      tgt.setSectViolation(true);
    }
  }

  // --------------------------------------------------------------------------------

  private void validateSems(Slot src, Slot tgt) {
    Sems srcSem = //
    src.getSectRef().getParentRef(). //
        getCourRef().getParentRef(). //
        getSemsRef().getParentRef();

    Sems tgtSem = //
    tgt.getSectRef().getParentRef(). //
        getCourRef().getParentRef(). //
        getSemsRef().getParentRef();

    if (srcSem == tgtSem) {
      src.setSemsViolation(true);
      tgt.setSemsViolation(true);
    }
  }
}
